import functools
import typing


def filtrate(key: str, value: str = None):
    """Method that allows to filter the events according
    to a set 'key', 'value'.

    Parameters
    ----------
    - key: required
        Key to be searched in the event.
    - value: optional
        Value needed in the last key.
    """

    def decorator(function: typing.Callable):
        @functools.wraps(function)
        def wrapper(message):
            if isinstance(message, dict):
                if key in message:
                    if key is None:
                        return function(message)
                    if message[key] == value:
                        return function(message)

        return wrapper

    return decorator



from typing import Callable
import logging
import pprint

from greenswitch import InboundESL
from greenswitch.esl import ESLEvent

from jaspion.sketch import Sketch


class Jaspion(Sketch, InboundESL):
    def __init__(self, host: str, port: int, password: str):
        """Method created to perform the call of the two __init __ 's
        present in the SuperClasses.
        """
        Sketch.__init__(self, __name__)
        InboundESL.__init__(self, host, int(port), password)

    def _safe_exec_handler(self, handler: Callable, event: ESLEvent):
        """Overridden method to ensure that the event passed to
        handlers is actually a dict instance with information
        already processed.

        Parameters
        ----------
        - handler: required
            Callable that will be called to process the event.
            The same should accept only a mandatory filing for
            no exceptions to occur.
        - event: required
            ESLEvent instance with date already parsed.
        """
        try:
            data = event.headers
            worker, client = handler

            if client:
                worker(self, data)

            else:
                worker(data)

        except Exception as exc:
            name = handler.__name__
            logging.exception("ESL %s raised exception." % name)
            logging.exception(pprint.pformat(data))
            logging.exception(exc)

    def command(self, command: str, background: bool = False):
        """Send command to freeswitch using api or bgapi.

        Parameters
        ----------
        - command: required
            Command to be forwarded to freeswitch.
        - background: not required
            Defines whether the freeswitch should run the
            background command or not.
        """
        preffix = "bgapi" if background else "api"
        result = super().send(f"{preffix} {command}").data

        return result.split(": ", 1)[-1] if background else result

    def process_events(self):
        """Overridden method to create the filters in the ESL
        before starting the processing of events.
        """
        for event in self.event_handlers.keys():
            if event.isupper():
                self.send("filter Event-Name {}".format(event))
            else:
                self.send("filter Event-Subclass {}".format(event))
        super().process_events()

    def start(self, *args, **kwargs):
        """Method created to be overwritten."""
        ...

    def run(self):
        """Method called to request the events.

        Raises
        ------
        - greenswitch.NotConnectedError:
            Connection failed while attempting to send command
            to FreeSwitch.
        """
        self.start()
        self.connect()
        self.send("events plain ALL")
        self.process_events()



import os
import importlib

import click
from greenswitch.esl import NotConnectedError

from jaspion import Jaspion


@click.group()
def main():
    """Jaspion CLI to manipulate and execute projects."""
    ...


@main.command()
@click.option(
    "--host",
    envvar="FSHOST",
    show_default=True,
    default="127.0.0.1",
    help="Address of FreeSwitch.",
)
@click.option(
    "--port",
    type=int,
    default=8021,
    envvar="FSPORT",
    show_default=True,
    help="Port to ESL connect.",
)
@click.option(
    "--password",
    default="ClueCon",
    show_default=True,
    envvar="FSPASSWD",
    help="Password to ESL connect.",
)
def runserver(host, port, password):
    """Connect in freeswitch and start a listner."""
    try:
        module = os.environ.get("JASPION_APP", None)
        sketch = "app"

        if ":" in module:
            module, sketch = module.split(":", 1)

        if module:
            click.echo("Try to connect in esl://{}:{}".format(host, port))
            mod = importlib.import_module(module)
            listner = getattr(mod, sketch)

            click.echo("Listner: {}".format(listner))
            app = Jaspion(host, port, password)
            app.update(listner)
            app.run()

        else:
            click.echo("No application configured.")

    except ImportError:
        click.echo(click.style("Failed to load listener.", fg="red"))

    except (KeyError, TypeError):
        click.echo(click.style("Invalid listener configured.", fg="red"))

    except (NotConnectedError, ConnectionRefusedError):
        click.echo(click.style("Failed to connect with freeswitch.", fg="red"))

    except KeyboardInterrupt:
        click.echo("Stoping...")
        app.stop()



from collections.abc import MutableMapping
from collections.abc import MutableSequence
from collections.abc import Callable
import functools
import reprlib
import typing


class Sketch(MutableMapping):
    def __init__(self, name: str):
        """Method used to save the Sketch name and create a
        dictionary instance in 'self.event_handlers'.
        """
        self.__name__ = name
        self.event_handlers = {}

    def __contains__(self, key: str):
        """Direct access interface to 'self.handlers'."""
        if key in self.event_handlers:
            return True
        return False

    def __eq__(self, other: object):
        """Direct access interface to 'self.handlers'."""
        if isinstance(other, self.__class__):
            return self.event_handlers == other.event_handlers
        else:
            return False

    def __ne__(self, other: object):
        """Direct access interface to 'self.handlers'."""
        return not self.__eq__(other)

    def __iter__(self):
        """Returns an iterable with the 'event_handlers' keys."""
        return (key for key in self.event_handlers.keys())

    def __len__(self):
        """Direct access interface to 'self.handlers'."""
        return len(self.event_handlers)

    def __getitem__(self, key: str):
        """Direct access interface to 'self.handlers'."""
        if key in self.event_handlers:
            return self.event_handlers[key]
        raise KeyError(key)

    def __setitem__(self, event: str, handler: typing.Callable):
        """Direct access interface to 'self.handlers'."""
        if isinstance(handler, Callable):
            self.event_handlers[event] = [handler]

        elif isinstance(handler, MutableSequence):
            self.event_handlers[event] = handler

        else:
            name = type(handler).__name__
            raise TypeError('"%s" not not is valid object' % name)

    def __delitem__(self, key: str):
        """Direct access interface to 'self.handlers'."""
        if key in self.event_handlers:
            del self.event_handlers[key]

    def keys(self):
        """Direct access interface to 'self.handlers'."""
        return self.event_handlers.keys()

    def values(self):
        """Direct access interface to 'self.handlers'."""
        return self.event_handlers.values()

    def items(self):
        """Direct access interface to 'self.handlers'."""
        return self.event_handlers.items()

    def popitem(self):
        """Direct access interface to 'self.handlers'."""
        if self.event_handlers:
            return self.event_handlers.popitem()
        name = type(self).__name__
        raise KeyError("popitem(): %s is empty" % name)

    def pop(self, key: str, default: typing.Any = None):
        """Direct access interface to 'self.handlers'."""
        if self.event_handlers:
            return self.event_handlers.pop(key, default)
        name = type(self).__name__
        raise KeyError("popitem(): %s is empty" % name)

    def setdefault(self, key: str, default: typing.Any = None):
        """Direct access interface to 'self.handlers'."""
        return self.event_handlers.setdefault(key, default)

    def update(self, other: object):
        """Method to be used to record Sketch's instance of
        Jaspion or even concatenated Sketch's.

        Parameters
        ----------
        - other: required
            An Sketch instance or another mutable mapping
            with callable values.

        Raises
        ------
        - TypeError:
            Object provided for update is invalid.
        """
        if not isinstance(other, Sketch):
            name = type(other).__name__
            raise TypeError("%s not is valid object." % name)

        for event, handlers in other.event_handlers.items():
            self.event_handlers.setdefault(event, []).extend(handlers)

    def clear(self):
        """Direct access interface to 'self.handlers'."""
        self.event_handlers.clear()

    def register_handle(
        self, event: str, function: typing.Callable, client: bool = False
    ):
        """Function used to add a function to 'event handlers'
        associated with the event reported.

        Parameters
        ----------
        - event: required
            Name of the event to be parsed.
        - function: required
            Function that will process the event.
        - client: not required
            Indicates whether or not the function expects to receive a
            client instance as a parameter. Useful for making reactive
            applications.
        """
        self.event_handlers.setdefault(event, []).append((function, client))

    def handle(self, event: str, client: bool = False) -> typing.Callable:
        """Decorator that allows the registration of new handlers.
        The event will be provided for the function in the form
        of a Dict.

        Parameters
        ----------
        - event: required
            Name of the event to be parsed.
        - client: not required
            Indicates whether or not the function expects to receive a
            client instance as a parameter. Useful for making reactive
            applications.
        """

        def decorator(function: typing.Callable):
            self.register_handle(event, function, client)

            @functools.wraps(function)
            def wrapper(*args, **kwargs):
                result = function(*args, **kwargs)
                return result

            return wrapper

        return decorator

    def __repr__(self):
        """Creates a valid representation for class
        and its subclasses.
        """
        name = self.__name__
        clss = type(self).__name__
        events = "[]"
        if self.event_handlers:
            repre = reprlib.repr(self.event_handlers.keys())
            key = repre.find("[") + 1
            events = repre[key:-2]

        return "%s(name=%s, events=%s)" % (clss, name, events)



from abc import abstractmethod
from typing import Dict
from abc import ABC


class AbstractBaseHandler(ABC):
    def __init__(self, event: Dict):
        self.event = event

        try:
            self.setup()
            self.handle()

        finally:
            self.finish()

    def setup(self):
        """Method called to start event processing. Can be overwritten."""
        ...

    @abstractmethod
    def handle(self):
        """Method called to process the event. Must be overwritten."""

    def finish(self):
        """Method to terminate processing. Can be overwritten."""
        ...

    def __repr__(self):
        """Generates representation for debug."""
        name = type(self).__name__

        if self.event["Event-Name"] == "CUSTOM":
            event = self.event["Event-Subclass"]
        else:
            event = self.event["Event-Name"]

        return "%s(%s)" % (name, event)



from setuptools import setup, find_packages
from os import path


here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="Jaspion",
    version="0.3.5.5",
    description="FreeSwitch Event Handler based in Flask.",
    include_package_data=True,
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Otoru/Jaspion",
    author="Vitor Hugo de Oliveira Vargas",
    author_email="vitor.hugo@sippulse.com",
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Telecommunications Industry",
        "Natural Language :: Portuguese (Brazilian)",
        "Operating System :: Unix",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Communications :: Telephony",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="ESL, FreeSwitch",
    packages=find_packages(exclude=["docs", "examples"]),
    install_requires=["greenswitch", "click"],
    extras_require={"dev": ["ipython", "pylint"]},
    entry_points={"console_scripts": ["jaspion=jaspion.cli:main"]},
    zip_safe=False,
    project_urls={
        "Bug Reports": "https://github.com/Otoru/Jaspion/issues",
        "Source Code": "https://github.com/Otoru/Jaspion",
        "Docs": "https://github.com/Otoru/jaspion/wiki",
    },
)
